// JavaVM.cpp --- Copyright (c) 1987-2009 by Design Science, Inc.
// purpose: manages a Sun JVM for MathFlow

#include "StdAfx.h"
#include "JavaVM.h"

// instantiate a global singleton instance of this class
CJavaVM theJVM; 

HRESULT
CJavaVM::InitVM(
	BSTR jrerootpath,
	BSTR classpath
) {
	// if we have already initialized the JVM just return
	if (m_pjvm)
		return S_FALSE;

	// set up dll paths
	USES_CONVERSION;

	// cache the jre root path for use in GetAWT
	// ensuring that there is no trailing backslash
	#pragma warning( disable : 4365 )
	const char* rootpath = OLE2CA(jrerootpath);
	#pragma warning( default : 4365 )

	Assert(strlen(rootpath) < 256);
	if (rootpath[strlen(rootpath) - 1] == '\\')
		strncpy_s(m_JREpath, rootpath, (strlen(rootpath) - 1));
	else 
		strcpy_s(m_JREpath, rootpath);

	char jvmpath[300];
	strcpy_s(jvmpath, m_JREpath);
	strcat_s(jvmpath, "\\bin\\client\\jvm.dll");

	// load the jvm.dll from specified path and call GetProcAddress
	HINSTANCE hVM = LoadLibraryA(jvmpath);
	if (hVM == NULL) {
		::MessageBoxA(NULL, "Couldn't load jvm.dll", "error", MB_OK);
		return E_FAIL;
	}
	#pragma warning( disable : 4191 )
	LPFNJVMFUNC pfnCreateJVM = 
		reinterpret_cast<LPFNJVMFUNC>(::GetProcAddress(hVM, "JNI_CreateJavaVM"));
	#pragma warning( default : 4191 )

	// Create the VM
	jint res;
	JNIEnv *env;
	#pragma warning( disable : 4365 )
	const char* cp = OLE2CA(classpath);
	#pragma warning( default : 4365 )

	#ifdef JNI_VERSION_1_2
		JavaVMInitArgs vm_args;
		JavaVMOption options[1];

		char cpstr[512];
		Assert(::strlen(cp) < 490);
		if (::strlen(cp) < 490) {
			::strcpy_s(cpstr, "-Djava.class.path=");
			::strcat_s(cpstr, cp);
		}

		options[0].optionString =  cpstr;
		vm_args.version = version;
		vm_args.options = options;
		vm_args.nOptions = 1;
		vm_args.ignoreUnrecognized = JNI_TRUE;
		// Create the Java VM 
		res = pfnCreateJVM(&m_pjvm, (void**)&env, &vm_args);
	#else
		::MessageBox(0, "MathFlow requires Java 1.2 or higher", "Error", MB_OK);		
	#endif // JNI_VERSION_1_2 

	if (res < 0) {
		::MessageBoxA(0, "Couldn't start Java VM", "Error", MB_OK);		
		return E_FAIL;
	}

	return S_OK;
}

HRESULT
CJavaVM::GetAWT(
	JNIEnv *env,
	JAWT *awtReturn
) {
	Assert(env);
	Assert(awtReturn);

	// must be called after InitVM, so that jrepath is set
	Assert(m_pjvm);
	if (!m_pjvm) {
		return E_FAIL;
	}

	// Load jawt.dll if necessary
	if (m_pfnGetAwt == NULL) {

		char awtpath[300];
		strcpy_s(awtpath, m_JREpath);
		strcat_s(awtpath, "\\bin\\jawt.dll");

		// Finally load the AWT dll from specified path and call GetProcAddress
		HINSTANCE hAWT = LoadLibraryA(awtpath);
		if (hAWT == NULL) {
			::MessageBoxA(NULL, "Couldn't load jawt.dll", "error", MB_OK);
			return E_FAIL;
		}
		// WARNING: Sun evidently doesn't declare this function as extern "C"
		// so we need to refer to a mangled name.  When changing versions of the
		// JVM bundled with MathFlow, we must take care to insure we use the right
		// name, as this is likely fragile.
		#pragma warning( disable : 4191 )
		m_pfnGetAwt = 
			reinterpret_cast<LPFNAWTFUNC>(::GetProcAddress(hAWT, "_JAWT_GetAWT@8"));
		#pragma warning( default : 4191 )
		if (m_pfnGetAwt == NULL) {
			::MessageBoxA(NULL, "Couldn't locate GetAWT function", "error", MB_OK);
			return E_FAIL;
		}	
	}
	
	jboolean success = m_pfnGetAwt(env, awtReturn);
	Assert(success != JNI_FALSE);

	return S_OK;
}

JavaVM *
CJavaVM::GetJavaVMPtr(
) {
	return m_pjvm;
}
